(**********************************************************************)
(*                                                                    *)
(* Function:    MHT- MEASURE HIGH TIME                                *)
(*                                                                    *)
(* Creation Date: 03/Mar/92                 From: NEW                 *)
(*                                                                    *)
(* Author:  Amy Dyson                                                 *)
(*                                                                    *)
(* Description:                                                       *)
(* ------------                                                       *)
(* MHT measures the high time of a wave using TCR1 and writes this to *)
(* a parameter in ram. After initialization, MHT will continuously    *)
(* measure every high time that comes in.  The high time input must   *)
(* be less than or equal to $FFFF TCR1 ticks.  MHT interrupts the CPU *)
(* after measuring each high time.                                    *)                                                           
(*                                                                    *)
(* Updates:   By:   Modification:                                     *)
(* --------   ---   -------------                                     *)
(* 11/Apr/93   JL   Converted to new syntax.                          *)
(*                                                                    *)
(*--------------------------------------------------------------------*)
(* Standard Exits Used:-   End_Of_Phase: N         End_Of_Link: Y     *)
(*                                                                    *)
(* External Files included: NONE                                      *)
(*                                                                    *)
(* CODE SIZE excluding standard exits = 14 LONG WORDS                 *)
(*--------------------------------------------------------------------*)
(*                                                                    *)
(*                                                                    *)
(**********              This Revision:  1.1                  *********)
(*                                                                    *)
(**********   LAST MODIFIED: 11/Apr/93       BY: Jeff Loeliger ********)
(*                                                                    *)
(**********************************************************************)
(***************************************************************************)
(*Motorola reserves the right to make changes without further notice to any*)
(*product herein. Motorola makes no warranty, representation or guarantee  *)
(*regarding the suitability of its products for any particular purpose, nor*)
(*does Motorola assume any liability arising out of the application or use *)
(*of any product or circuit, and specifically disclaims any and all        *)
(*liability, including without limitation consequential or incidental      *)
(*damages. "Typical" parameters can and do vary in different applications. *)
(*All operating parameters, including "Typical",must be validated for each *)
(*customer application by customer's technical experts. Motorola does not  *)
(*convey any license under its patent rights nor the rights of others.     *)
(*Motorola products are not designed, intended, or authorized for use as   *)
(*components in systems intended for surgical implant into the body, or    *)
(*other applications intended to support or sustain life, or for any other *)
(*application in which the failure of the Motorola product could create a  *)
(*situation where injury or death may occur. Should Buyer purchase or use  *)
(*Motorola products for any such unintended or unauthorized application,   *)
(*Buyer, shall indemnify and hold Motorola and its officers, employees,    *)
(*subsidiaries, affiliates, and distributors harmless against all claims,  *)
(*costs, damages, and expenses, and reasonable attorney fees arising out   *)
(*of, directly or indirectly, any claim of personal injury or death        *)
(*associated with such unintended or unauthorized use, even if such claim  *)
(*alleges that Motorola was negligent regarding the design or manufacture  *)
(*of the part.                                                             *)
(*Motorola and the Motorola logo are registered trademarks of Motorola Inc.*) 
(*Motorola is an Equal Opportunity/Affirmative Action Employer.            *)
(*Copyright Motorola Inc. 1993                                             *)
(***************************************************************************)


(*()()()()()()()()()()() DATA STRUCTURE ()()()()()()()()()()()()()()()*)
(*                                                                    *)
(* name:               Written By:            Location  Bits:         *)
(* -----               -----------            ---------------         *)
(* START_TIME_MHT          TPU                 Parameter0  0..15      *)
(*                     Start time of a high pulse.  MHT uses this     *)
(*                     to measure high times.                         *)
(* HIGH_TIME_MHT           TPU                 Parameter1   0..15     *)                       
(*                     High time of the last high pulse measured.     *)
(*                                                                    *)
(* hsr1   hsr0         Action                                         *)
(* ----   ----         ------                                         *)
(*  1       1          Initialize MHT function                        *)
(*                                                                    *)
(*                                                                    *)
(* Links Accepted: NO              Links Generated: NO                *)
(*                                                                    *)
(* Interrupts Generated After:     Every high time is measured        *)
(*                                                                    *)
(*()()()()()()()()()()()()()()()()()()()()()()()()()()()()()()()()()()*)


(*+++++++++++++++++++++  PARAMETER MACROS  +++++++++++++++++++++++++++*)

%macro START_TIME_MHT        'prm0'.
%macro HIGH_TIME_MHT         'prm1'.

(*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*)


(*====================================================================*)
(*|||||||||||||||||||||  MICROCODE STARTS BELOW  |||||||||||||||||||||*)
(*VVVVVVVVVVVVVVVVVVVVV--------------------------VVVVVVVVVVVVVVVVVVVVV*)


(**********************************************************************)
(*                                                                    *)
(* ENTRY name:  INIT_MHT                                              *)
(*                                                                    *)
(* STATE(S) ENTERED: S1                                               *)
(*                                                                    *)
(* PRELOAD PARAMETER : none                                           *)
(*                                                                    *)
(* ENTER WHEN : HSR = %11                                             *)
(*                                                                    *)
(* ACTION: chan set time base as TCR1, set pac to look for low to high*)
(*         transition.  Negate latches and enable m/tsr requests.     *)
(*                                                                    *)
(**********************************************************************)
%entry name = INIT_MHT; start_address *; disable_match; 
cond hsr1=1,hsr0=1,lsr=x,m/tsr=x,pin=x,flag0=x; 
ram p <- none.          

        chan    TBS:= in_m1_c1,          (* pin is input, use TCR1 *)
                PAC:= low_high,          (* wait for low to high trans *)
                enable_mtsr.             (* enable m/tsr requests *)
        chan    neg_mrl, neg_tdl, neg_lsl;      (* negate latches *)
                end.                            (* end of S1 *)

(**********************************************************************)
(*                                                                    *)
(* ENTRY name:  TRANS_HIGH_MHT                                        *)
(*                                                                    *)
(* STATE(S) ENTERED: S2                                               *)
(*                                                                    *)
(* PRELOAD PARAMETER : none                                           *)
(*                                                                    *)
(* ENTER WHEN : HSR = %00 m/tsr=1 lsr=X flag0=X pin=1                 *)
(*                                                                    *)
(* ACTION: Store ert (start time of high pulse) in ram.  set pac      *)
(*         to high_low to wait for low edge.  Negate tdl.             *)
(*                                                                    *)
(**********************************************************************)
%entry name = TRANS_HIGH_MHT; start_address *; disable_match; 
cond hsr1=0,hsr0=0,lsr=x,m/tsr=1,pin=1,flag0=x; 
ram p <- none.         

        au      diob:= ert;                  (* load start time in diob *)
        ram     diob -> @START_TIME_MHT.     (* store start time in ram *)
        chan    PAC:= high_low;              (* wait for high to low trans *)
                neg_tdl;                     (* negate tdl *)    
                end.                         (* end of S2 *)

(**********************************************************************)
(*                                                                    *)
(* ENTRY name:  TRANS_LOW_MHT                                         *)
(*                                                                    *)
(* STATE(S) ENTERED: S3                                               *)
(*                                                                    *)
(* PRELOAD PARAMETER : START_TIME_MHT                                 *)
(*                                                                    *)
(* ENTER WHEN : HSR = %00 m/tsr=1 lsr=X flag0=X pin=0                 *)
(*                                                                    *)
(* ACTION: Fetch start time from ram.  Subtract start time from       *)
(*         end time to get high time.  Store high time in ram.        *)
(*         set pac to low_high to wait for next high pulse.  Negate   *)
(*         tdl.  Interrupt CPU to indicate a new high pulse has just  *)
(*         been measured.                                             *)
(*                                                                    *)
(**********************************************************************)
%entry name = TRANS_LOW_MHT; start_address *; disable_match; 
cond hsr1=0,hsr0=0,lsr=x,m/tsr=1,pin=0,flag0=x; 
ram p <- @START_TIME_MHT.

        au      diob:= ert-p;                (* subtract start time from *)
                                             (* end time to get high time *)
        ram     diob -> @HIGH_TIME_MHT.      (* store high time in ram *)
        chan    PAC:= low_high,              (* wait for low to high trans *)
                neg_tdl;                     (* negate tdl *)    
                chan cir;                    (* interrupt CPU to indicate *)
                                             (* new pulse measure *)
                end.                         (* end of S3 *)


(**********************************************************************)
(*                       UNUSED STATES                                *)
(**********************************************************************)

%entry name = UNUSED_MHT; start_address END_OF_LINK; disable_match; 
cond hsr1=0,hsr0=1,lsr=x,m/tsr=x,pin=x,flag0=x; 
ram p <-none.          

%entry name = UNUSED_MHT; start_address END_OF_LINK; disable_match; 
cond hsr1=1,hsr0=0,lsr=x,m/tsr=x,pin=x,flag0=x; 
ram p <-none.          

%entry name = UNUSED_MHT; start_address END_OF_LINK; disable_match; 
cond hsr1=0,hsr0=0,lsr=1,m/tsr=0,pin=x,flag0=x; 
ram p <-none.          
